from .core import add_acast

try:
    import omegaconf
    from omegaconf import OmegaConf
    
    add_acast(omegaconf.listconfig.ListConfig, object, OmegaConf.to_object)
    add_acast(omegaconf.listconfig.ListConfig, list, OmegaConf.to_container)
    
except ImportError:
    pass
